/**
 * BNSim is an open-source, parallel, stochastic, and multi-scale modeling
 * platform which integrates various simulation algorithms, together with
 * genetic circuits and chemotaxis pathway models in a 3D environment.
 *
 * BNSim is developed by CMU SLD Group, and released under GPLv2 license
 * Please check http://www.ece.cmu.edu/~sld/bnsim for more information
 *
 * BNSim is developed using C++ and pthread under Linux and Mac OS
 *
 * If you use it for your work, please cite
 * Wei, Guopeng, Paul Bogdan, and Radu Marculescu. "Efficient Modeling and 
 * Simulation of Bacteria-Based Nanonetworks with BNSim." Selected Areas in
 * Communications, IEEE Journal on 31.12 (2013): 868-878.
 *
 * @author      Guopeng (Daniel) Wei  danielwei@cmu.edu
 * @version     2.5
 * @since       1.0
 */

#include "core.h"
#include <cstring>

using namespace BNSim;
using namespace std;


int main(int argn, char ** argv) {


    Sim::param.WorldSize = new myVector3d(1000,1000,1000);
	Sim::param.cubeNumber.x = Sim::param.cubeNumber.z = Sim::param.cubeNumber.y = 20;
	Sim::param.ThreadNumber = 2;
	Sim::param.ChemicalNumber = 1;
    Sim::param.workdir = "/Users/guopengwei/Documents/Chetest";
	Sim::param.bacNum= 1000;
    Sim::param.timeStep = 0.01;
    Sim::param.bacteriaRadius = 2;
    Sim::param.quorumSensing = false;
    Sim::param.chemotaxis  = true;
    Sim::param.RecordVisit = true;
     for (int i=1; i<argn; i++) {

        if (strcmp(argv[i], "-directory") == 0) {
            Sim::param.workdir = argv[++i];
            continue;
        }

        if (strcmp(argv[i], "-bacteriaRadius") == 0) {
            if (!sscanf(argv[++i], "%d", &Sim::param.bacteriaRadius)) {
                    return 1;
            }
            continue;
        }
     }

    cout<<"Configuring environment..."<<endl;

    // enable chemotaxis
    Sim::param.chemotaxis = true;
    Universe * myuniv =  new Universe();
	Sim::univ = myuniv;

    cout<<"Adding bacteria..."<<endl;

	// add bacteria

	double velocity = 14;   // um/s

	for(int i = 0; i != Sim::param.bacNum; ++i) {
		myVector3d Position;
		Position.pos.x = 10 + 200*((double) rand() / (RAND_MAX));
		Position.pos.y = 10 + 200*((double) rand() / (RAND_MAX));
		Position.pos.z = 10 + 200*((double) rand() / (RAND_MAX));

		Bacteria* bac = new Bacteria(Position, Sim::param.bacteriaRadius, velocity, i);

		// Two bacteria cannot occupy the same space
		if(Sim::param.bacteriaRadius!= 0 && bac->intersect()) {
			delete bac; i--;
		}
		else {
			myuniv->addBacteria(bac);
			bac->registerinCube(myuniv->getCube(bac->getCubeLoc()),bac->getCurrentCubeRelativeIndex());
		}
	}

	Sim::univ->prepare_multithreading();
    Sim::param.bacteriaOnly = true;   // we use a fixed gradient in this example (see tick() in bacteria/chemotaxis.cpp), no need to diffuse chemicals here

    // calibrate the intracellular chemotaxis pathway for 300 seconds
    Sim::param.calibrate = true;

    for(int t = 0;t<300.0/Sim::param.timeStep;t++) {
        myuniv->tick();
        double per = 100*(t/(300.0/Sim::param.timeStep));
        cout<<"Calibration Percentage: "<<per<<endl;
    }

    Sim::param.calibrate = false;

    int dumpStep = 0;

    NodeCoverageExporter NCE;

    NCE.update();

    double percentage = 0;
    exporter::dump_Bacteria();
    int bacHit = 0;
    
    cout<<"Simulation start"<<endl;

	// Main simulation loop, simulation length, 900 seconds
	for(;Sim::param.time<1000000;Sim::param.time+=Sim::param.timeStep,dumpStep++) {
		myuniv->tick();
      
        if(dumpStep == 50000) {

            dumpStep = 0; bacHit = 0;

            exporter::dump_Bacteria();

            if(percentage<0.98)
            	percentage = NCE.update();
           
            for (int i= 0; i!=Sim::param.bacNum; ++i) {
                if(Sim::BacteriaList[i]->getHittingTime()!=-1)  bacHit++;
            }

            if(bacHit > 0.98*Sim::param.bacNum) break;
            
            cout<<"Simulation time: "<< Sim::param.time << " s bachit: "<<bacHit<<endl;
        }
		
	}

    exporter::dump_HittingTime();
    cout<<"Simulation done"<<endl;

}
