/**
 * BNSim is an open-source, parallel, stochastic, and multi-scale modeling
 * platform which integrates various simulation algorithms, together with
 * genetic circuits and chemotaxis pathway models in a 3D environment.
 *
 * BNSim is developed by CMU SLD Group, and released under GPLv2 license
 * Please check http://www.ece.cmu.edu/~sld/bnsim for more information
 *
 * BNSim is developed using C++ and pthread under Linux and Mac OS
 *
 * If you use it for your work, please cite
 * Wei, Guopeng, Paul Bogdan, and Radu Marculescu. "Efficient Modeling and
 * Simulation of Bacteria-Based Nanonetworks with BNSim." Selected Areas in
 * Communications, IEEE Journal on 31.12 (2013): 868-878.
 *
 * @author      Guopeng (Daniel) Wei  danielwei@cmu.edu
 * @version     2.5
 * @since       1.0
 */

#include <cstdlib>
#include "../core/core.h"

using namespace BNSim;
using namespace std;

// see model description in Jiang, Lili, Qi Ouyang, and Yuhai Tu. "Quantitative modeling of Escherichia coli chemotactic motion in environments varying in space and time." PLoS computational biology 6.4 (2010): e1000735.

ChemotaxisSystem::ChemotaxisSystem(Bacteria * host) {
    this->host = host;
    CCW = false;
    Y = 0;
    Ya = 5.33;
    Ye = 3; //um
    H = 10;
    inverseTau = 5;
    kr = 0.2;
    kb = 0.2;
    m = 2;
    m0 = 1;
    alpha = 1.7;
    N_tar = 6;
    aspcon = 0;
    activity = 0.5;
    KA = 3;
    KI = 0.0182;
    Kd = 3.06;
    g0 = 40;
    g1 = 40;
    w = 1.3;
    tau = 0;
    runtime = 0;
    tumbletime = 1;
}

double ChemotaxisSystem::randomUniform() {
	return ((double) rand() / (RAND_MAX));
}

void ChemotaxisSystem::tick() {
    aspcon = 1e-4*(host->getPos()->length());
//
	if(Sim::param.calibrate) {
		double kr = 0.01, kb = 0.01;
		m = m + (kr*(1-activity)-kb*activity);
	}
	else {
		m = m + Sim::param.timeStep*(kr*Sim::param.timeStep*(1-activity)-kb*Sim::param.timeStep*activity);
	}

	double fm = alpha*(m0-m);
	double epsilon = fm - log((1+ aspcon/KA)/(1+aspcon/KI));
	activity = 1/(1+exp(N_tar*epsilon));

    Y = Ya*activity;
    
    if(CCW && tau<runtime) {
        tau += Sim::param.timeStep;
    }
    else if(CCW && tau>runtime) {
        tumbletime = log(1/randomUniform())/kPlus();
        tau = 0;
        CCW = false;
    }
    else if(!CCW && tau<tumbletime) {
        tau += Sim::param.timeStep;
    }
    else if(!CCW && tau>tumbletime) {
        runtime = log(1/randomUniform())/kMinus();
        tau = 0;
        CCW = true;
    }
}

/*
 Sneddon, Michael W., William Pontius, and Thierry Emonet. "Stochastic coordination of multiple actuators reduces latency and improves chemotactic response in bacteria." Proceedings of the National Academy of Sciences 109.3 (2012): 805-810.
 */

// rate from CW to CCW
double ChemotaxisSystem::kPlus() {
    double r = w*exp((g0/4.0) - (g0/2.0)*( Y/(Kd+Y))) ;
    return r;
}

// rate from CCW to CW
double ChemotaxisSystem::kMinus() {
    double r = w*exp(-(g1/4.0) + (g1/2.0)*( Y/(Kd+Y))) ;
    return r;
}
