/**
 * BNSim is an open-source, parallel, stochastic, and multi-scale modeling
 * platform which integrates various simulation algorithms, together with
 * genetic circuits and chemotaxis pathway models in a 3D environment.
 *
 * BNSim is developed by CMU SLD Group, and released under GPLv2 license
 * Please check http://www.ece.cmu.edu/~sld/bnsim for more information
 *
 * BNSim is developed using C++ and pthread under Linux and Mac OS
 *
 * If you use it for your work, please cite
 * Wei, Guopeng, Paul Bogdan, and Radu Marculescu. "Efficient Modeling and
 * Simulation of Bacteria-Based Nanonetworks with BNSim." Selected Areas in
 * Communications, IEEE Journal on 31.12 (2013): 868-878.
 *
 * @author      Guopeng (Daniel) Wei  danielwei@cmu.edu
 * @version     2.5
 * @since       1.0
 */


#include "core.h"

using namespace BNSim;
using namespace std;

Universe::Universe() {
	// first calculate the geometry of each cube
	Sim::param.cubeSize.x = Sim::param.WorldSize->pos.x/Sim::param.cubeNumber.x;
	Sim::param.cubeSize.y = Sim::param.WorldSize->pos.y/Sim::param.cubeNumber.y;
	Sim::param.cubeSize.z = Sim::param.WorldSize->pos.z/Sim::param.cubeNumber.z;

	for(int i = 0; i!=Sim::param.cubeNumber.x*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z; ++i)
		Sim::cubesList.push_back(new EnvCube());

	// initialize the Sim::cubesList

	for(int i = 0; i < Sim::param.cubeNumber.x; i++) {
		for(int j = 0; j < Sim::param.cubeNumber.y; j++) {
			for(int k = 0; k < Sim::param.cubeNumber.z; k++) {
				for(unsigned int p = 0; p < Sim::param.ChemicalNumber; p++) {
					Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->chemicals.push_back(0);
					Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->preChemicals.push_back(0);
				}
				Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->particleNumber = 0;
                Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->mut.unlock();
				if(Sim::param.bacteriaRadius !=0) {
					int index = 0, sites = Sim::param.cubeSize.x*Sim::param.cubeSize.y*Sim::param.cubeSize.z/(Sim::param.bacteriaRadius*Sim::param.bacteriaRadius*Sim::param.bacteriaRadius);            // suppose bacteria is Sim::param.bacteriaRadius*Sim::param.bacteriaRadius*Sim::param.bacteriaRadius sphere
					while(index++<sites) {
						Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->occupancy.push_back(0);
						if(Sim::param.RecordVisit)
							Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->visited.push_back(0);
					}
				}
			}
		}
	}

	// prepare multithreading
	thr = new pthread_t[Sim::param.ThreadNumber]; // 1 for the collision
	thr_data = new thread_data_t[Sim::param.ThreadNumber];
	evn_thr_data = new thread_data_t[Sim::param.ThreadNumber];
    
//    if(Sim::param.networkExport)
//        BacNetwork = new BacteriaNetwork();
}

Universe::~Universe() {
	cout<<"[I] removing the whole universe..."<<endl;

	for(int i = 0; i < Sim::param.cubeNumber.x; i++) {
		for(int j = 0; j < Sim::param.cubeNumber.y; j++) {
			for(int k = 0; k < Sim::param.cubeNumber.z; k++) {
				Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->chemicals.clear();
				Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->preChemicals.clear();
				Sim::cubesList[i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k]->occupancy.clear();
			}
		}
	}
	Sim::cubesList.clear();

    for(unsigned int i=0;i<Sim::BacteriaList.size();i++)
    	delete Sim::BacteriaList[i];
    Sim::BacteriaList.clear();
}

EnvCube* Universe::getCube(const intVector3d cubeLocation) {
	return Sim::cubesList[cubeLocation.x*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + cubeLocation.y*Sim::param.cubeNumber.z + cubeLocation.z];
}

EnvCube* Universe::getCube(int x, int y, int z) {
	if(x>=0 && x<Sim::param.cubeNumber.x && y>=0 && y<Sim::param.cubeNumber.y && z>=0 && z<Sim::param.cubeNumber.z)
		return Sim::cubesList[x*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + y*Sim::param.cubeNumber.z + z];
	else
		return NULL;
}

EnvCube* Universe::getCube(int index) {
	return Sim::cubesList[index];
}

Bacteria * Universe::getBacteria(int index) {
	return Sim::BacteriaList[index];
}

const intVector3d Universe::getcubeLocation(const myVector3d* location){
	intVector3d loc;

	double x = location->pos.x/(double)(Sim::param.cubeSize.x);
	double y = location->pos.y/(double)(Sim::param.cubeSize.y);
	double z = location->pos.z/(double)(Sim::param.cubeSize.z);

	double remainderx = (int)(x*100) % 100;
	double remaindery = (int)(y*100) % 100;
	double remainderz = (int)(z*100) % 100;
	if(remainderx>=50)		loc.x = floor(x)+1;
	else 	loc.x = floor(x);
	if(remaindery>=50)		loc.y = floor(y)+1;
	else 	loc.y = floor(y);
	if(remainderz>=50)		loc.z = floor(z)+1;
	else 	loc.z = floor(z);

	if(loc.x == Sim::param.cubeNumber.x) loc.x  = Sim::param.cubeNumber.x-1;
	if(loc.y == Sim::param.cubeNumber.y) loc.y  = Sim::param.cubeNumber.y-1;
	if(loc.z == Sim::param.cubeNumber.z) loc.z  = Sim::param.cubeNumber.z-1;

	return loc;
}

void Universe::addBacteria(Bacteria * bac) {
	Sim::BacteriaList.push_back(bac);
}

/* thread function */
void * environment_thread(void *arg) {
		thread_data_t *data = (thread_data_t *)arg;
		int startIndex = data->start;
		int endIndex = data->end;

  // update previous value
		for(unsigned int p = 0; p < Sim::param.ChemicalNumber; p++) {
			for(int i=startIndex;i<endIndex;i++)
				for(int j=0;j<Sim::param.cubeNumber.y;j++)
					for(int k=0;k<Sim::param.cubeNumber.z;k++) {
						Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->preChemicals[p] = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p];
						Sim::chemical[p].totalamount = 0;
					}
		}


  		int xAbove, xBelow, yAbove, yBelow, zAbove, zBelow;
  		// Sim::univ->Sim::cubesList of chemical leaving the cube in the positive (negative) .. direction
  		double qxAbove, qxBelow, qyAbove, qyBelow, qzAbove, qzBelow;

  		/**
  		 * Flux of molecules crossing in the positive x-direction (Fick's law)
  		 * 	J = -D(dC/dx) = -D*(C(x+dx)-C(x))/dx =  -D*(N(x+dx)-N(x))/((dx)^2*dy*dz)  molecules/(micron)^2/sec
  		 * Number of molecules transferred in the positive x-direction over dt
  		 * 	xAbove = J*(dy*dz)*dt = -((D*dt)/(dx)^2)*(N(x+dx)-N(x)) = -kX*(N(x+dx)-N(x))
  		 * where kX = (D*dt)/(dx)^2 is a dimensionless constant
  		 */
  		double normX =  Sim::param.timeStep/pow(Sim::param.cubeSize.x,2);
  		double normY =  Sim::param.timeStep/pow(Sim::param.cubeSize.y,2);
  		double normZ =  Sim::param.timeStep/pow(Sim::param.cubeSize.z,2);



  		double temp = 0;

  		for(unsigned int p = 0; p < Sim::param.ChemicalNumber; p++) {

  			double kX = Sim::chemical[p].diffusionCoefficient*normX;	double kY = Sim::chemical[p].diffusionCoefficient*normY;	double kZ = Sim::chemical[p].diffusionCoefficient*normZ;

  			double totalamount = 0;
  			// diffusion

			for(int i=startIndex;i<endIndex;i++)
				for(int j=0;j<Sim::param.cubeNumber.y;j++)
					for(int k=0;k<Sim::param.cubeNumber.z;k++) {

						xAbove = (i == Sim::param.cubeNumber.x-1 ? -1 : i+1);		xBelow = (i == 0 ? -1 : i-1);
						yAbove = (j == Sim::param.cubeNumber.y-1 ? -1 : j+1);		yBelow = (j == 0 ? -1 : j-1);
						zAbove = (k == Sim::param.cubeNumber.z-1 ? -1 : k+1);		zBelow = (k == 0 ? -1 : k-1);

						if(xAbove != -1) {
							/* Calculate the Sim::cubesList of chemical leaving the box in this direction */
							qxAbove = -kX*(Sim::univ->getCube(xAbove*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+k)->preChemicals[p]-Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->preChemicals[p]);
							temp = Sim::univ->getCube(xAbove*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+k)->chemicals[p]+qxAbove;
							/* Add that Sim::cubesList to the box above */
							Sim::univ->getCube(xAbove*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+k)->chemicals[p]=(temp>0?temp:0);
							/* Remove it from this box */
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qxAbove;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}
						else if(Sim::param.leakyBoundary) {
							qxAbove = kX*Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p];
							/* Remove it from this box */
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qxAbove;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}

						if(xBelow != -1) {
							/* Calculate the Sim::cubesList of chemical leaving the box in this direction */
							qxBelow = -kX*(Sim::univ->getCube(xBelow*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+k)->chemicals[p]-Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]);
							temp = Sim::univ->getCube(xBelow*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+k)->chemicals[p]+qxBelow;
							/* Add that Sim::cubesList to the box above */
							Sim::univ->getCube(xBelow*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+k)->chemicals[p]=(temp>0?temp:0);
							/* Remove it from this box */
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qxBelow;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}
						else if(Sim::param.leakyBoundary) {
							qxBelow = kX*Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p];
							/* Remove it from this box */
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qxBelow;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}

						if(yAbove != -1) {
							qyAbove = -kY*(Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +yAbove*Sim::param.cubeNumber.z+k)->chemicals[p]-Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]);
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +yAbove*Sim::param.cubeNumber.z+k)->chemicals[p]+qyAbove;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +yAbove*Sim::param.cubeNumber.z+k)->chemicals[p]=(temp>0?temp:0);
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qyAbove;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}
						else if(Sim::param.leakyBoundary) {
							qyAbove = kY*Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p];
							/* Remove it from this box */
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qyAbove;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}

						if(yBelow != -1) {
							qyBelow = -kY*(Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +yBelow*Sim::param.cubeNumber.z+k)->chemicals[p]-Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]);
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +yBelow*Sim::param.cubeNumber.z+k)->chemicals[p]+qyBelow;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +yBelow*Sim::param.cubeNumber.z+k)->chemicals[p]=(temp>0?temp:0);
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qyBelow;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}
						else if(Sim::param.leakyBoundary) {
							qyBelow = kY*Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p];
							/* Remove it from this box */
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qyBelow;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}

						if(zAbove != -1) {
							qzAbove = -kZ*(Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+zAbove)->chemicals[p]-Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]);
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+zAbove)->chemicals[p]+qzAbove;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+zAbove)->chemicals[p]=(temp>0?temp:0);
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qzAbove;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}
						else if(Sim::param.leakyBoundary) {
							qzAbove = kZ*Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p];
							/* Remove it from this box */
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qzAbove;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}

						if(zBelow != -1) {
							qzBelow = -kZ*(Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+zBelow)->chemicals[p]-Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]);
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+zBelow)->chemicals[p]+qzBelow;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z +j*Sim::param.cubeNumber.z+zBelow)->chemicals[p]=(temp>0?temp:0);
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qzBelow;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}
						else if(Sim::param.leakyBoundary) {
							qzBelow = kZ*Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p];
							/* Remove it from this box */
							temp = Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]-qzBelow;
							Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]=(temp>0?temp:0);
						}

						Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p] -= (Sim::param.timeStep*Sim::chemical[p].degradationRate*Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]);
						if(Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p]<0) Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p] =0;

						totalamount += Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[p];
					}
  		}
  	  pthread_exit(NULL);
}

void * bacteria_thread(void *arg){
	  thread_data_t *data = (thread_data_t *)arg;
	  for(unsigned int i=data->start; i<data->end; i++) {
          if(Sim::univ->getBacteria(i)->isalive())
			  Sim::univ->getBacteria(i)->tick();
	  }
	  pthread_exit(NULL);
}

void Universe::tick(void) {
	if(Sim::param.bacteriaOnly)
		update_bacteria_parallel();
	else if(Sim::param.environmentOnly)
		update_environment_p();
	else {
		update_bacteria_parallel();
		update_environment_p();
	}
}

void Universe::update_bacteria_parallel(void) {
	  unsigned int i;int rc;
	  // tick
	  for (i = 0; i < Sim::param.ThreadNumber; ++i) {
		if ((rc = pthread_create(&thr[i], NULL, bacteria_thread, &thr_data[i]))) {
		  return;
		}
	  }
	  /* block until all threads complete */
	  for (i = 0; i < Sim::param.ThreadNumber; ++i) {
		pthread_join(thr[i], NULL);
	  }
}

void Universe::update_environment_p(void) {
	  unsigned int i;int rc;
	  // tick Ecoli

	  for (i = 0; i < Sim::param.ThreadNumber; ++i) {
		if ((rc = pthread_create(&thr[i], NULL, environment_thread,  &evn_thr_data[i]))) {
		  return;
		}
	  }

	  /* block until all threads complete */
	  for (i = 0; i < Sim::param.ThreadNumber; ++i) {
		pthread_join(thr[i], NULL);
	  }
}

void Universe::prepare_multithreading(void) {
	 unsigned int threadstep = Sim::BacteriaList.size()/Sim::param.ThreadNumber;
	 unsigned int envthreadstep = Sim::param.cubeNumber.x/Sim::param.ThreadNumber;

	 // prepare data-structure
	  unsigned int i;
	  for (i = 0; i < Sim::param.ThreadNumber-1; ++i) {
		thr_data[i].start = i*threadstep;
		thr_data[i].end = (i+1)*threadstep;

		evn_thr_data[i].start = i*envthreadstep;
		evn_thr_data[i].end = (i+1)*envthreadstep;

	  }
	  thr_data[Sim::param.ThreadNumber-1].start = thr_data[Sim::param.ThreadNumber-2].end;
	  thr_data[Sim::param.ThreadNumber-1].end = Sim::BacteriaList.size();

	  evn_thr_data[Sim::param.ThreadNumber-1].start =  evn_thr_data[Sim::param.ThreadNumber-2].end;
	  evn_thr_data[Sim::param.ThreadNumber-1].end = Sim::param.cubeNumber.x;
}

int Universe::checkChemical(string name) {

	for(unsigned int i=0;i!=Sim::chemical.size();++i) {
		if(Sim::chemical[i].name ==name)
			return i;
	}

	return -1;
}

void Universe::addChemical(int index, double con) {
	for(int i=0;i<Sim::param.cubeNumber.x;i++)
				for(int j=0;j<Sim::param.cubeNumber.y;j++)
					for(int k=0;k<Sim::param.cubeNumber.z;k++) {
						Sim::univ->getCube(i*Sim::param.cubeNumber.y*Sim::param.cubeNumber.z + j*Sim::param.cubeNumber.z + k)->chemicals[index] = con;
					}
}


